@@ -23,7 +23,7 @@ import os
 
 from apport.hookutils import attach_related_packages, attach_file_if_exists
 
-from ubuntuone.platform.xdg_base_directory import ubuntuone_log_dir
+from ubuntuone.platform.logger import ubuntuone_log_dir
 
 
 def add_info(report):
