@@ -23,7 +23,9 @@ import os
 from collections import defaultdict
 from functools import wraps
 
-from twisted.internet.defer import inlineCallbacks, returnValue
+from twisted.internet.defer import (inlineCallbacks,
+                                    returnValue,
+                                    DeferredLock)
 from ubuntuone.platform.credentials import CredentialsManagementTool
 
 from ubuntuone.controlpanel import sd_client, replication_client
@@ -124,9 +126,11 @@ class ControlBackend(object):
         UPLOAD_KEY: -1,  # no limit
     }
 
+    credentials_lock = DeferredLock()
+
     def __init__(self, shutdown_func=None):
         """Initialize the web_client."""
-        self._status_changed_handler = None
+        self._status_changed_handlers = []
         self._credentials = None
         self._volumes = {}  # cache last known volume info
 
@@ -135,6 +139,7 @@ class ControlBackend(object):
 
         self.login_client = CredentialsManagementTool()
         self.sd_client = sd_client.SyncDaemonClient()
+        self.is_sd_client_handler_set = False
         self.wc = WebClient(self.get_credentials)
 
         logger.info('ControlBackend: instance started.')
@@ -171,7 +176,7 @@ class ControlBackend(object):
         is_synching = bool(status['is_connected'])
         is_idle = bool(status['is_online']) and status['queues'] == 'IDLE'
         is_disconnected = status['name'] == 'WAITING' or \
-                          (status['name'] == 'READY' and \
+                          (status['name'] == 'READY' and
                            'Not User' in status['connection'])
         is_starting = status['name'] in ('INIT', 'LOCAL_RESCAN', 'READY')
         is_stopped = status['name'] == 'SHUTDOWN'
@@ -198,24 +203,27 @@ class ControlBackend(object):
         else:
             return result
 
-    def _set_status_changed_handler(self, handler):
-        """Set 'handler' to be called when file sync status changes."""
-        logger.debug('setting status_changed_handler to %r', handler)
-
-        def process_and_callback(status):
-            """Process syncdaemon's status and callback 'handler'."""
-            result = self._process_file_sync_status(status)
+    def process_status_and_call_handlers(self, status):
+        """Process syncdaemon's status and call each of the handlers."""
+        result = self._process_file_sync_status(status)
+        for handler in self._status_changed_handlers:
             handler(result)
 
-        self._status_changed_handler = handler
-        self.sd_client.set_status_changed_handler(process_and_callback)
+    def add_status_changed_handler(self, handler):
+        """Add 'handler' to list of callbacks for file sync status changes."""
 
-    def _get_status_changed_handler(self):
-        """Return the handler to be called when file sync status changes."""
-        return self._status_changed_handler
+        if handler in self._status_changed_handlers:
+            logger.debug('Handler %r already in list of handlers, ignoring.' %
+                         handler)
+            return
 
-    status_changed_handler = property(_get_status_changed_handler,
-                                      _set_status_changed_handler)
+        logger.debug('adding %r to list of status_changed_handlers', handler)
+
+        self._status_changed_handlers.append(handler)
+        if not self.is_sd_client_handler_set:
+            cb = self.process_status_and_call_handlers
+            self.sd_client.set_status_changed_handler(cb)
+            self.is_sd_client_handler_set = True
 
     @inlineCallbacks
     def _process_device_web_info(self, devices,
@@ -323,8 +331,12 @@ class ControlBackend(object):
     @inlineCallbacks
     def get_credentials(self):
         """Find credentials."""
-        if not self._credentials:
-            self._credentials = yield self.login_client.find_credentials()
+        yield self.credentials_lock.acquire()
+        try:
+            if not self._credentials:
+                self._credentials = yield self.login_client.find_credentials()
+        finally:
+            self.credentials_lock.release()
         returnValue(self._credentials)
 
     @inlineCallbacks
@@ -820,6 +832,55 @@ class ControlBackend(object):
         """Restore the file sync settings."""
         yield self.change_file_sync_settings(self.DEFAULT_FILE_SYNC_SETTINGS)
 
+    @log_call(logger.info)
+    @inlineCallbacks
+    def sync_menu(self):
+        """Obtain the data to create the sync menu."""
+        result = yield self.sd_client.sync_menu()
+        returnValue(result)
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def get_public_files(self):
+        """Trigger the action to get the public files."""
+        yield self.sd_client.get_public_files()
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def set_public_files_list_handler(self, handler):
+        """Return the handler to be called for the public files list."""
+        result = yield self.sd_client.set_public_files_list_handler(handler)
+        returnValue(result)
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def get_shares(self):
+        """Get the information of the shares."""
+        result = yield self.sd_client.get_shares()
+        returnValue(result)
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def change_public_access(self, path, is_public):
+        """Change the type access of a file."""
+        yield self.sd_client.change_public_access(path, is_public)
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def set_public_access_changed_handler(self, handler):
+        """Return the handler to be called when a access type change."""
+        result = yield self.sd_client.set_public_access_changed_handler(
+            handler)
+        returnValue(result)
+
+    @log_call(logger.info)
+    @inlineCallbacks
+    def set_public_access_change_error_handler(self, handler):
+        """Return the handler to be called on access type change error."""
+        result = yield self.sd_client.set_public_access_change_error_handler(
+            handler)
+        returnValue(result)
+
     @log_call(logger.info)
     def shutdown(self):
         """Stop this service."""
