@@ -278,8 +278,7 @@ class ControlPanelBackend(dbus.service.Object):
     @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
     def file_sync_status(self):
         """Get the status of the file sync service."""
-        if self.backend.status_changed_handler is None:
-            self.backend.status_changed_handler = self.process_status
+        self.backend.add_status_changed_handler(self.process_status)
 
         d = self.backend.file_sync_status()
         d.addCallback(self.process_status)
