@@ -33,13 +33,13 @@ from ubuntuone.controlpanel.gui.qt import handle_errors
 logger = setup_logging('qt.addfolder')
 
 CLOSE = QtGui.QMessageBox.Close
-# NOTE: this is temporary because of a Qt bug that will be fixed
-# on Qt 4.9. You cannot, in general, use sys.platform in these
+# NOTE: this is temporary because of Qt bugs.
+# You cannot, in general, use sys.platform in these
 # modules. Do not do this. Please. This is an exception, one
 # time only, pinky-swear, cross my heart and hope to die if
 # I lie.
 FILE_CHOOSER_OPTIONS = QtGui.QFileDialog.ShowDirsOnly
-if sys.platform == 'win32':
+if sys.platform in ('win32', 'darwin'):
     FILE_CHOOSER_OPTIONS |= QtGui.QFileDialog.DontUseNativeDialog
 # Yes, that is true. If you do that again, you will be hunted
 # down and taught a lesson. You will be sorry.
