@@ -35,6 +35,7 @@ from ubuntuone.controlpanel.gui import (
     MAIN_DEVICES_TAB,
     MAIN_FOLDERS_TAB,
     MAIN_PREFERENCES_TAB,
+    MAIN_SHARE_LINKS_TAB,
     PERCENTAGE_LABEL,
     show_quota_warning,
     TALK_TO_US,
@@ -75,6 +76,9 @@ class ControlPanel(UbuntuOneBin):
 
         self.ui.tab_widget.setTabText(
             self.ui.tab_widget.indexOf(self.ui.folders_tab), MAIN_FOLDERS_TAB)
+        self.ui.tab_widget.setTabText(
+            self.ui.tab_widget.indexOf(self.ui.share_links_tab),
+                MAIN_SHARE_LINKS_TAB)
         self.ui.tab_widget.setTabText(
             self.ui.tab_widget.indexOf(self.ui.devices_tab), MAIN_DEVICES_TAB)
         self.ui.tab_widget.setTabText(
