@@ -26,6 +26,8 @@ import threading
 from PyQt4 import QtGui, QtCore
 from twisted.internet import defer
 
+from ubuntuone.platform import is_link
+
 from ubuntuone.controlpanel.utils import default_folders
 from ubuntuone.controlpanel.logger import setup_logging, log_call
 from ubuntuone.controlpanel.gui import (
@@ -38,6 +40,7 @@ from ubuntuone.controlpanel.gui import (
     FOLDERS_COLUMN_NAME,
     FOLDERS_COLUMN_SYNC_LOCALLY,
     FOLDERS_CONFIRM_MERGE,
+    FOLDER_EXISTS_AS_FILE,
     FOLDERS_MANAGE_LABEL,
     GET_MORE_STORAGE,
     humanize,
@@ -80,10 +83,15 @@ YES = QtGui.QMessageBox.Yes
 
 def _process_name(name):
     """Tweak 'name' with a translatable music folder name."""
-    if name == MUSIC_REAL_PATH:
+    if name.startswith('~/'):
+        suggested = name[2:]
+    else:
+        suggested = name
+
+    if suggested == MUSIC_REAL_PATH:
         result = MUSIC_DISPLAY_NAME
     else:
-        result = name
+        result = suggested
     return result
 
 
@@ -132,6 +140,9 @@ class FoldersPanel(UbuntuOneBin):
         headers.setResizeMode(SUBSCRIPTION_COL, headers.ResizeToContents)
         headers.setResizeMode(EXPLORE_COL, headers.ResizeToContents)
 
+        self.ui.folders.setSortingEnabled(True)
+        self.ui.folders.sortByColumn(FOLDER_NAME_COL, QtCore.Qt.AscendingOrder)
+
         self.ui.share_publish_button.setText(FOLDERS_MANAGE_LABEL)
         self.ui.share_publish_button.uri = MANAGE_FILES_LINK
         QtGui.QApplication.instance().focusChanged.connect(self.focus_changed)
@@ -203,7 +214,8 @@ class FoldersPanel(UbuntuOneBin):
                     child.volume_path = volume['path']
                 child.volume_id = volume['volume_id']
 
-                name = _process_name(volume[u'display_name'])
+                name = _process_name(volume.get(u'suggested_path',
+                    volume[u'display_name']))
                 child.setText(FOLDER_NAME_COL, name)
                 child.setToolTip(FOLDER_NAME_COL, name)
                 child.setToolTip(EXPLORE_COL, FOLDERS_COLUMN_EXPLORE)
@@ -319,16 +331,23 @@ class FoldersPanel(UbuntuOneBin):
                     os.path.exists(volume_path))
         response = YES
         if subscribed and os.path.exists(volume_path):
-            text = FOLDERS_CONFIRM_MERGE % {'folder_path': volume_path}
-            buttons = YES | NO | CANCEL
-            response = QtGui.QMessageBox.warning(self, '', text, buttons, YES)
+            if os.path.isdir(volume_path) and not is_link(volume_path):
+                text = FOLDERS_CONFIRM_MERGE % {'folder_path': volume_path}
+                buttons = YES | NO | CANCEL
+                response = QtGui.QMessageBox.warning(self, '', text, buttons,
+                    YES)
+            else:
+                text = FOLDER_EXISTS_AS_FILE % {'folder_path': volume_path}
+                buttons = CLOSE
+                QtGui.QMessageBox.warning(self, '', text, buttons)
+                response = NO
 
         self.is_processing = True
 
         if response == YES:
             # user accepted, merge the folder content
             yield self.backend.change_volume_settings(volume_id,
-                                                {'subscribed': subscribed})
+                                            {'subscribed': subscribed})
             self.load()
         else:
             # restore old value
@@ -417,14 +436,24 @@ class LocalFoldersPanel(UbuntuOneBin):
     """The panel that only shows local, non-synched folders."""
 
     changesApplied = QtCore.pyqtSignal()
-    changesCanceled = QtCore.pyqtSignal()
     logger = logger
     ui_class = local_folders_ui
 
     def __init__(self, *args, **kwargs):
+        timer = kwargs.pop('timer', None)
+        if timer is None:
+            timer = QtCore.QTimer()
+
+        queue = kwargs.pop('queue', None)
+        if queue is None:
+            queue = Queue.Queue()
+
         super(LocalFoldersPanel, self).__init__(*args, **kwargs)
-        self.queue = Queue.Queue()
-        self.timer = QtCore.QTimer()
+
+        self.queue = queue
+        self.timer = timer
+        self.timer.timeout.connect(self.update_sizes)
+
         self.items = {}
         self.user_home = None
         self.account_info = None
@@ -437,8 +466,7 @@ class LocalFoldersPanel(UbuntuOneBin):
 
         headers = self.ui.folders.header()
         headers.setResizeMode(LOCAL_SUBSCRIPTION_COL, headers.Stretch)
-        headers.setResizeMode(LOCAL_SPACE_COL, headers.Custom)
-        headers.resizeSection(LOCAL_SPACE_COL, 130)
+        headers.setResizeMode(LOCAL_SPACE_COL, headers.ResizeToContents)
 
         self.ui.folders.headerItem().setText(LOCAL_SUBSCRIPTION_COL,
                                              LOCAL_FOLDERS_FOLDER_HEADER)
@@ -467,7 +495,6 @@ class LocalFoldersPanel(UbuntuOneBin):
     def _stop(self):
         """Stop all pending threads and timers."""
         self.timer.stop()
-        self.timer = None
 
         for item in self.items.itervalues():
             if item.thread is None:
@@ -481,6 +508,7 @@ class LocalFoldersPanel(UbuntuOneBin):
     def load(self):
         """Load specific tab info."""
         self.is_processing = True
+        self.user_home = yield self.backend.get_home_dir()
         self.account_info = yield self.backend.account_info()
         self._set_space_header(self.account_info['quota_used'])
         volumes_info = yield self.backend.volumes_info(with_storage_info=False)
@@ -496,28 +524,28 @@ class LocalFoldersPanel(UbuntuOneBin):
                 for volume in volumes:
                     if (volume[u'type'] == self.backend.FOLDER_TYPE and
                         bool(volume['subscribed'])):
-                        folders.append((volume['path'], volume['volume_id']))
+                        suggested = volume.get(u'suggested_path',
+                            volume[u'display_name'])
+                        folders.append((volume['path'], volume['volume_id'],
+                                        suggested))
 
             # add local folders only if they are valid
-            self.user_home = yield self.backend.get_home_dir()
             for folder in default_folders(user_home=self.user_home):
                 is_valid = yield self.backend.validate_path_for_folder(folder)
                 if is_valid:
-                    folders.append((folder, None))
+                    folders.append((folder, None, None))
 
             # always clear the items dict first, since clearing the folders
             # list will trigger "underlying C/C++ object has been deleted"
             self.items.clear()
             self.ui.folders.clear()
 
-            # self.timer can be None if self._stop was called before
-            # this piece of code was executed
-            if self.timer is not None:
-                for path, volume_id in folders:
-                    self.add_folder(folder_path=path, volume_id=volume_id)
+            for path, volume_id, suggested in folders:
+                self.add_folder(folder_path=path, volume_id=volume_id,
+                    suggested=suggested)
 
+            if folders and not self.timer.isActive():
                 self.timer.start(2000)
-                self.timer.timeout.connect(self.update_sizes)
         finally:
             self.is_processing = False
 
@@ -547,19 +575,23 @@ class LocalFoldersPanel(UbuntuOneBin):
         self.changesApplied.emit()
 
     @handle_errors(logger=logger)
-    def add_folder(self, folder_path, volume_id=None):
+    def add_folder(self, folder_path, volume_id=None, suggested=None):
         """Add a folder to the list."""
         if folder_path in self.items:
             logger.warning('LocalFoldersPanel: already have an item for %r.',
                            folder_path)
+            return
 
-        if self.user_home is None:
+        if suggested is None:
             logger.warning('LocalFoldersPanel: user home is None! '
                            'paths will not be pretty.')
-            display_name = folder_path
-        else:
-            user_home = self.user_home + os.path.sep
+            if self.user_home is None:
+                user_home = ''
+            else:
+                user_home = self.user_home + os.path.sep
             display_name = _process_name(folder_path.replace(user_home, ''))
+        else:
+            display_name = _process_name(suggested)
 
         item = FolderItem([display_name, ""], path=folder_path,
                           queue=self.queue, volume_id=volume_id)
@@ -630,7 +662,8 @@ class LocalFoldersPanel(UbuntuOneBin):
     def on_folders_itemChanged(self, item, column):
         """Update the size for the chosen row."""
         if column == LOCAL_SUBSCRIPTION_COL:
-            self.update_sizes()
+            if item.volume_id is None:
+                self.update_sizes()
             self.items[item.path] = item
 
     # pylint: enable=C0103
