@@ -67,7 +67,7 @@ class MainWindow(QtGui.QMainWindow):
 
     def switch_to(self, tabname="folders"):
         """Switch control panel to the required tab."""
-        tabnames = ["folders", "devices", "settings", "account"]
+        tabnames = ["folders", "share_links", "devices", "settings", "account"]
         if tabname in tabnames:
             self.ui.control_panel.ui.tab_widget.setCurrentIndex(
                 tabnames.index(tabname))
@@ -119,14 +119,15 @@ def start(close_callback, minimized=False, with_icon=False, installer=False):
             window = MainWindow(installer=installer)
         else:
             window = MainWindow(close_callback=close_callback,
-                installer=installer)
+                                installer=installer)
         app = QtGui.QApplication.instance()
         style = QtGui.QStyle.alignedRect(
-                    QtCore.Qt.LeftToRight, QtCore.Qt.AlignCenter,
-                    window.size(), app.desktop().availableGeometry())
+            QtCore.Qt.LeftToRight, QtCore.Qt.AlignCenter,
+            window.size(), app.desktop().availableGeometry())
         window.setGeometry(style)
         window.check_updates()
         window.show()
+        window.raise_()
     else:
         window = None
     if with_icon or minimized:
