@@ -17,6 +17,7 @@
 """Provide the correct ui main module."""
 
 import argparse
+import os
 import sys
 
 from PyQt4 import QtCore
@@ -29,14 +30,16 @@ from ubuntuone.controlpanel.gui.qt.ui import images_rc
 from ubuntuone.controlpanel.gui.qt.uniqueapp import UniqueApplication
 
 # Invalid name "source", pylint: disable=C0103
-if sys.platform == 'win32':
-    from ubuntuone.controlpanel.gui.qt.main import windows
-    source = windows
+if sys.platform in ('win32', 'darwin'):
+    from ubuntuone.controlpanel.gui.qt.main import twisted_main
+    source = twisted_main
 else:
-    from ubuntuone.controlpanel.gui.qt.main import linux
-    source = linux
+    from ubuntuone.controlpanel.gui.qt.main import dbus_main
+    source = dbus_main
 # pylint: enable=C0103
 
+from ubuntuone.controlpanel.utils import install_config_and_daemons
+
 
 def parser_options():
     """Parse command line parameters."""
@@ -44,8 +47,8 @@ def parser_options():
     result.add_argument("--switch-to", dest="switch_to",
                         metavar="PANEL_NAME", default="",
                         help="Start Ubuntu One in the "
-                             "PANEL_NAME tab. Possible values are: "
-                             "folders, devices, settings, account")
+                            "PANEL_NAME tab. Possible values are: "
+                            "folders, share_links, devices, settings, account")
     result.add_argument("--minimized", dest="minimized", action="store_true",
                         default=False,
                         help="Start Ubuntu One "
@@ -61,14 +64,37 @@ def parser_options():
     return result
 
 
-def main(args):
+def install_qt4reactor():
+    """Import and install the qt4reactor."""
+    import qt4reactor
+    qt4reactor.install()
+
+
+def main(args, install_reactor_darwin=False):
     """Start the Qt mainloop and open the main window."""
+
+    # Disable the overlay-scrollbar GTK module that was
+    # added in Ubuntu 12.10 because it breaks Qt (LP:1007421)
+    gtk_mod = os.getenv('GTK_MODULES')
+    if gtk_mod is not None:
+        gtk_mod = ':'.join([mod for mod in
+            gtk_mod.split(':') if mod != 'overlay-scrollbar'])
+        os.environ['GTK_MODULES'] = gtk_mod
+
     # The following cannot be imported outside this function
     # because u1trial already provides a reactor.
 
     args = ['ubuntuone-installer'] + args
+    if sys.platform == 'darwin':
+        args += ['-graphicssystem', 'raster']
     app = UniqueApplication(args, "ubuntuone-control-panel")
 
+    # on darwin, must install qt4reactor after UniqueApplication init.
+    # otherwise qt4reactor will create a QCoreApplication for us,
+    # instead of the QApplication we need
+    if install_reactor_darwin and sys.platform == 'darwin':
+        install_qt4reactor()
+
     # Install translator for standard dialogs.
     locale = unicode(QtCore.QLocale.system().name())
     translator = QtCore.QTranslator()
@@ -84,7 +110,9 @@ def main(args):
     args = parser.parse_args(args=arg_list[bin_position:])
     switch_to = args.switch_to
     minimized = args.minimized
-    with_icon = args.with_icon
+    # On Darwin, because of how apps are started, we need
+    # this by default.
+    with_icon = args.with_icon or sys.platform == 'darwin'
     installer = args.installer
     source.main(app)
 
@@ -97,6 +125,8 @@ def main(args):
         data.append(unicode(qss.data()))
     app.setStyleSheet('\n'.join(data))
 
+    install_config_and_daemons()
+
     # Unused variable 'window', 'icon', pylint: disable=W0612
     icon, window = start(lambda: source.main_quit(app),
                          minimized=minimized, with_icon=with_icon,
