@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
-
-# Authors: Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2011 Canonical Ltd.
+# Copyright 2011-2012 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -31,7 +29,6 @@ from ubuntuone.controlpanel.gui import (
     SETTINGS_ALLOW_NOTIFICATIONS,
     SETTINGS_AUTO_CONNECT,
     SETTINGS_BANDWIDTH,
-    SETTINGS_BANDWIDTH_ZERO_WARNING,
     SETTINGS_BUTTON_APPLY,
     SETTINGS_BUTTON_DEFAULT,
     SETTINGS_FILE_SYNC,
@@ -41,6 +38,7 @@ from ubuntuone.controlpanel.gui import (
     SETTINGS_SYNC_ALL_FOLDERS,
     SETTINGS_SYNC_ALL_SHARES,
 )
+from ubuntuone.controlpanel.gui.qt import force_wordwrap
 from ubuntuone.controlpanel.gui.qt.ubuntuonebin import UbuntuOneBin
 from ubuntuone.controlpanel.gui.qt.ui import preferences_ui
 
@@ -56,22 +54,6 @@ def check_state(checked):
     return CHECKED if checked else UNCHECKED
 
 
-def tweak_speed(speed):
-    """Return the proper speed in kilobytes.
-
-    If speed < 0, return -1.
-    If speed == 0, return 1 (almost nothing, the backed won't accept 0.
-    If speed > 0, return the kilobytes presentation of that.
-
-    """
-    result = -1
-    if speed > 0:
-        result = speed * KILOBYTES
-    elif speed == 0:
-        result = 1
-    return result
-
-
 class PreferencesPanel(UbuntuOneBin):
     """The Preferences Tab Panel widget"""
 
@@ -82,19 +64,13 @@ class PreferencesPanel(UbuntuOneBin):
         """Do some extra setupping for the UI."""
         super(PreferencesPanel, self)._setup()
         self.ui.apply_changes_button.setText(SETTINGS_BUTTON_APPLY)
-        self.ui.autoconnect_checkbox.setText(SETTINGS_AUTO_CONNECT)
         self.ui.bandwidth_settings.setTitle(SETTINGS_BANDWIDTH)
         self.ui.file_sync_settings.setTitle(SETTINGS_FILE_SYNC)
-        self.ui.kbps_label_1.setText(SETTINGS_KILOBITS_PER_SECOND)
-        self.ui.kbps_label_2.setText(SETTINGS_KILOBITS_PER_SECOND)
-        self.ui.label_2.setText(SETTINGS_BANDWIDTH_ZERO_WARNING)
-        self.ui.limit_uploads_checkbox.setText(SETTINGS_LIMIT_UPLOAD)
-        self.ui.limit_downloads_checkbox.setText(SETTINGS_LIMIT_DOWNLOAD)
+        self.ui.upload_speed_spinbox.setSuffix("  " +
+            SETTINGS_KILOBITS_PER_SECOND)
+        self.ui.download_speed_spinbox.setSuffix("  " +
+            SETTINGS_KILOBITS_PER_SECOND)
         self.ui.restore_defaults_button.setText(SETTINGS_BUTTON_DEFAULT)
-        self.ui.share_autosubscribe_checkbox.setText(SETTINGS_SYNC_ALL_FOLDERS)
-        self.ui.show_all_notifications_checkbox.setText(
-            SETTINGS_ALLOW_NOTIFICATIONS)
-        self.ui.udf_autosubscribe_checkbox.setText(SETTINGS_SYNC_ALL_SHARES)
 
     # pylint: disable=E0202
     @defer.inlineCallbacks
@@ -125,40 +101,18 @@ class PreferencesPanel(UbuntuOneBin):
         # signal handlers
         self.ui.download_speed_spinbox.setValue(download_speed)
         self.ui.upload_speed_spinbox.setValue(upload_speed)
+        if upload_speed > 0:
+            self.ui.limit_uploads_checkbox.setCheckState(CHECKED)
+        else:
+            self.ui.limit_uploads_checkbox.setCheckState(UNCHECKED)
+        if download_speed > 0:
+            self.ui.limit_downloads_checkbox.setCheckState(CHECKED)
+        else:
+            self.ui.limit_downloads_checkbox.setCheckState(UNCHECKED)
         self.is_processing = False
 
-    def process_speed_settings(self, new_value, checkbox):
-        """A new speed limit was set."""
-        if new_value < 0:
-            checkbox.setCheckState(UNCHECKED)
-        else:
-            checkbox.setCheckState(CHECKED)
-
-    # Invalid name "on_{down, up}load_speed_spinbox_valueChanged"
     # pylint: disable=C0103
 
-    @QtCore.pyqtSlot(int)
-    def on_download_speed_spinbox_valueChanged(self, new_value):
-        """A new download speed was set."""
-        self.process_speed_settings(new_value,
-                                    self.ui.limit_downloads_checkbox)
-
-    @QtCore.pyqtSlot(int)
-    def on_upload_speed_spinbox_valueChanged(self, new_value):
-        """A new upload speed was set."""
-        self.process_speed_settings(new_value,
-                                    self.ui.limit_uploads_checkbox)
-
-    def on_limit_downloads_checkbox_stateChanged(self, new_state):
-        """Limit download speed checkbox changed."""
-        if new_state == UNCHECKED:
-            self.ui.download_speed_spinbox.setValue(-1)
-
-    def on_limit_uploads_checkbox_stateChanged(self, new_state):
-        """Limit upload speed checkbox changed."""
-        if new_state == UNCHECKED:
-            self.ui.upload_speed_spinbox.setValue(-1)
-
     @QtCore.pyqtSlot()
     @defer.inlineCallbacks
     def on_apply_changes_button_clicked(self):
@@ -167,8 +121,14 @@ class PreferencesPanel(UbuntuOneBin):
         notifications = self.ui.show_all_notifications_checkbox.checkState()
         share_autosubscribe = self.ui.share_autosubscribe_checkbox.checkState()
         udf_autosubscribe = self.ui.udf_autosubscribe_checkbox.checkState()
-        download_speed = tweak_speed(self.ui.download_speed_spinbox.value())
-        upload_speed = tweak_speed(self.ui.upload_speed_spinbox.value())
+        if self.ui.limit_uploads_checkbox.checkState() == UNCHECKED:
+            upload_speed = -1
+        else:
+            upload_speed = self.ui.upload_speed_spinbox.value() * KILOBYTES
+        if self.ui.limit_downloads_checkbox.checkState() == UNCHECKED:
+            download_speed = -1
+        else:
+            download_speed = self.ui.download_speed_spinbox.value() * KILOBYTES
 
         settings = {
             backend.AUTOCONNECT_KEY: autoconnect == CHECKED,
@@ -192,3 +152,22 @@ class PreferencesPanel(UbuntuOneBin):
         settings = yield self.backend.file_sync_settings_info()
 
         self.process_info(settings)
+
+    def resizeEvent(self, event):
+        super(PreferencesPanel, self).resizeEvent(event)
+        size_bandwidth = (self.ui.bandwidth_settings.width() -
+            self.ui.upload_speed_spinbox.width())
+        padding = 160  # Padding to allow shrinking
+        size_sync = (self.ui.file_sync_settings.width() - padding)
+        force_wordwrap(self.ui.limit_uploads_checkbox, size_bandwidth,
+            SETTINGS_LIMIT_UPLOAD)
+        force_wordwrap(self.ui.limit_downloads_checkbox, size_bandwidth,
+            SETTINGS_LIMIT_DOWNLOAD)
+        force_wordwrap(self.ui.autoconnect_checkbox, size_sync,
+            SETTINGS_AUTO_CONNECT)
+        force_wordwrap(self.ui.udf_autosubscribe_checkbox, size_sync,
+            SETTINGS_SYNC_ALL_SHARES)
+        force_wordwrap(self.ui.share_autosubscribe_checkbox, size_sync,
+            SETTINGS_SYNC_ALL_FOLDERS)
+        force_wordwrap(self.ui.show_all_notifications_checkbox, size_sync,
+            SETTINGS_ALLOW_NOTIFICATIONS)
