@@ -25,7 +25,11 @@ from ubuntuone.devtools.handlers import MementoHandler
 
 from ubuntuone.controlpanel import backend, cache
 from ubuntuone.controlpanel.tests import TestCase, EXPECTED_ACCOUNT_INFO, TOKEN
-from ubuntuone.controlpanel.gui import qt
+from ubuntuone.controlpanel.gui import (
+    qt,
+    RECENTTRANSFERS,
+    UPLOADING,
+)
 from ubuntuone.controlpanel.gui.tests import (
     FakedObject,
     FAKE_VOLUMES_INFO,
@@ -119,9 +123,11 @@ class FakedControlPanelBackend(FakedObject):
     next_result = []
     exposed_methods = [
         'account_info',
+        'add_status_changed_handler',
         'build_signed_iri',
         'change_device_settings',
         'change_file_sync_settings',
+        'change_public_access',
         'change_replication_settings',
         'change_volume_settings',
         'connect_files',
@@ -133,12 +139,16 @@ class FakedControlPanelBackend(FakedObject):
         'enable_files',
         'file_sync_settings_info',
         'file_sync_status',
+        'get_public_files',
         'login',
         'register',
         'remove_device',
         'replications_info',
         'restart_files',
         'restore_file_sync_settings',
+        'set_public_access_changed_handler',
+        'set_public_access_change_error_handler',
+        'set_public_files_list_handler',
         'shutdown',
         'start_files',
         'stop_files',
@@ -164,6 +174,17 @@ class FakedControlPanelBackend(FakedObject):
     def build_signed_iri(self, iri):
         """Fake iri signing."""
 
+    def sync_menu(self):
+        """Fake sync_menu."""
+        data = {}
+        data[RECENTTRANSFERS] = []
+        data[UPLOADING] = []
+        return data
+
+    def get_shares(self):
+        """Fake get_shares."""
+        return []
+
 
 class CrashyBackendException(Exception):
     """A faked backend crash."""
@@ -364,3 +385,16 @@ class BaseTestCase(TestCase):
         """The backend instance is correct."""
         if getattr(self.ui, 'backend', None) is not None:
             self.assertIsInstance(self.ui.backend, FakedControlPanelBackend)
+
+
+class FakeDesktopService(object):
+    """Fake QDesktopService."""
+
+    def __init__(self):
+        self.opened_url = None
+
+    # pylint: disable=C0103
+
+    def openUrl(self, url):
+        """Fake openUrl."""
+        self.opened_url = url
