@@ -24,7 +24,7 @@ import Queue
 import shutil
 import sys
 
-from PyQt4 import QtGui
+from PyQt4 import QtCore, QtGui
 from twisted.internet import defer
 from ubuntuone.devtools.handlers import MementoHandler
 from ubuntuone.devtools.testcases import skipIfOS
@@ -34,12 +34,16 @@ from ubuntuone.controlpanel.gui.tests import (
     FAKE_VOLUMES_INFO,
     FAKE_VOLUMES_MINIMAL_INFO,
     FAKE_VOLUMES_NO_FREE_SPACE_INFO,
-    MUSIC_FOLDER, ROOT, USER_HOME,
+    FAKE_VOLUMES_ONLY_ROOT_INFO,
+    MUSIC_FOLDER, MUSIC_PATH, ROOT, USER_HOME,
 )
+from ubuntuone.controlpanel.gui import MUSIC_REAL_PATH
 from ubuntuone.controlpanel.gui.qt import folders as gui
 from ubuntuone.controlpanel.gui.qt.tests import (
     BaseTestCase,
+    FakedControlPanelBackend,
     FakedDialog,
+    SAMPLE_ACCOUNT_INFO as ACCOUNT_INFO,
 )
 from ubuntuone.controlpanel.gui.qt.tests.test_ubuntuonebin import (
     UbuntuOneBinTestCase,
@@ -49,6 +53,11 @@ from ubuntuone.controlpanel.gui.qt.tests.test_ubuntuonebin import (
 # Instance of 'ControlBackend' has no '_called' member
 # pylint: disable=W0212, E1103
 
+DEFAULT_FOLDERS = [u'Music', u'Configuración', 'An invalid folder',
+    os.path.join(u'I ♥ Ubuntu', u'Documents'),
+    os.path.join(u'I ♥ Ubuntu', u'Something', u'Long'),
+]
+
 
 def volumes_with_music_unsubscribed():
     """Return a copy of FAKE_VOLUMES_MINIMAL_INFO with music unsubscribed."""
@@ -66,6 +75,49 @@ def _build_name(name):
     return name
 
 
+def size_for_path(path):
+    """Fake size calulation for paths."""
+    return sum(map(ord, path)) * len(path) * 10000
+
+
+class FakedCalculateSize(object):
+    """A faked CalculateSize thread."""
+
+    next_size = None
+
+    def __init__(self, path, queue):
+        self.path = path
+        self.queue = queue
+        self.started = False
+
+    def start(self):
+        """Fake start."""
+        self.started = True
+        if self.next_size is None:
+            size = size_for_path(self.path)
+        else:
+            size = self.next_size
+        self.queue.put([self.path, size])
+
+
+class FakedTimer(gui.QtCore.QTimer):
+    """A faked timer."""
+
+    def __init__(self, *args, **kwargs):
+        super(FakedTimer, self).__init__(*args, **kwargs)
+        self._active = False
+        # pylint: disable=C0103
+        self.isActive = lambda: self._active
+
+    def start(self, interval):
+        """Override start."""
+        self._active = True
+
+    def stop(self):
+        """Override stop."""
+        self._active = False
+
+
 class ExploreFolderButtonTestCase(BaseTestCase):
     """Test the ExploreFolderButton widget."""
 
@@ -74,8 +126,7 @@ class ExploreFolderButtonTestCase(BaseTestCase):
 
     def test_text(self):
         """The button text is correct."""
-        self.assertEqual(unicode(self.ui.text()),
-            gui.FOLDERS_COLUMN_EXPLORE)
+        self.assertEqual(unicode(self.ui.text()), gui.FOLDERS_COLUMN_EXPLORE)
 
     def test_clicked(self):
         """Clicking the button opens the folder in the default file manager."""
@@ -193,28 +244,21 @@ class FoldersPanelVolumesInfoTestCase(FoldersPanelTestCase):
 
     def test_process_info(self):
         """The volumes info is processed when ready."""
-        self.ui.process_info(FAKE_VOLUMES_INFO)
         folders = self.ui.ui.folders
-
+        self.ui.process_info(FAKE_VOLUMES_INFO)
         root = folders.invisibleRootItem()
         self.assertEqual(len(FAKE_VOLUMES_INFO), root.childCount())
 
-        treeiter = gui.QtGui.QTreeWidgetItemIterator(folders)
         for name, _, volumes in FAKE_VOLUMES_INFO:
             name = _build_name(name)
-
-            # get first child, matching "My Folders"
-            item = treeiter.value()
-            self.assertFalse(item.is_empty)
-
+            items = folders.findItems(name, QtCore.Qt.MatchFixedString
+                | QtCore.Qt.MatchCaseSensitive | QtCore.Qt.MatchRecursive,
+                gui.FOLDER_NAME_COL)
+            self.assertEqual(1, len(items))
+            item = items[0]
             self.assert_folder_group_header_correct(item, name)
-
             self.assertEqual(len(volumes), item.childCount())
-            sorted_vols = sorted(volumes, key=operator.itemgetter('path'))
-            for volume in sorted_vols:
-                treeiter += 1
-                item = treeiter.value()  # get child folder
-                self.assertTrue(item is not None)
+            for volume in volumes:
 
                 name = volume['path'].replace(USER_HOME + os.path.sep, '')
                 expected_path = volume['path']
@@ -227,12 +271,14 @@ class FoldersPanelVolumesInfoTestCase(FoldersPanelTestCase):
                 else:
                     icon_name = gui.SHARE_ICON_NAME
 
+                items = folders.findItems(name, QtCore.Qt.MatchFixedString |
+                    QtCore.Qt.MatchCaseSensitive | QtCore.Qt.MatchRecursive,
+                    gui.FOLDER_NAME_COL)
+                self.assertEqual(1, len(items))
+                item = items[0]
                 self.assert_folder_row_correct(item, name, icon_name, volume,
                                                tweaked_path=expected_path)
 
-            treeiter += 1
-            item = treeiter.value()
-
     def test_process_info_clears_the_list(self):
         """The old volumes info is cleared before updated."""
         self.ui.process_info(FAKE_VOLUMES_INFO)
@@ -254,9 +300,10 @@ class FoldersPanelVolumesInfoTestCase(FoldersPanelTestCase):
 
         child_index = 0
         root = self.ui.ui.folders.invisibleRootItem()
-        for name, _, _ in FAKE_VOLUMES_NO_FREE_SPACE_INFO:
-            name = _build_name(name)
-
+        names = [_build_name(name) for name, _, _ in
+            FAKE_VOLUMES_NO_FREE_SPACE_INFO]
+        names.sort(key=lambda x: x.lower())
+        for name in names:
             item = root.child(child_index)
             self.assertFalse(item.is_empty)
             self.assert_folder_group_header_correct(item, name)
@@ -363,35 +410,23 @@ class FoldersPanelVolumesInfoTestCase(FoldersPanelTestCase):
         """Ensure that the inner widgets are in the correct tab order."""
         self.ui.process_info(FAKE_VOLUMES_INFO)
         folders = self.ui.ui.folders
-
         widget = self.ui.ui.folders.nextInFocusChain()
         treeiter = gui.QtGui.QTreeWidgetItemIterator(folders)
-        for name, _, volumes in FAKE_VOLUMES_INFO:
-            item = treeiter.value()
-            sorted_vols = sorted(volumes, key=operator.itemgetter('path'))
-            for volume in sorted_vols:
+        while treeiter.value():
+            if treeiter.value().parent() is None:  # Top-levels
                 treeiter += 1
-                item = treeiter.value()  # get child folder
-
-                name = volume['path'].replace(USER_HOME + os.path.sep, '')
-                if volume['type'] == self.ui.backend.SHARE_TYPE:
-                    name = volume['name']
-                self.assertEqual(unicode(item.text(gui.FOLDER_NAME_COL)), name)
-
-                if volume['type'] != self.ui.backend.ROOT_TYPE:
-                    self.assertIsInstance(widget, QtGui.QCheckBox)
-                    self.assertEqual(unicode(
-                        self.ui.widget_items[widget].text(0)), name)
-                    widget = widget.nextInFocusChain()
-
-                if not self.ui.remote_folders:
-                    self.assertIsInstance(widget, QtGui.QPushButton)
-                    self.assertEqual(unicode(
-                        self.ui.widget_items[widget].text(0)), name)
-                    widget = widget.nextInFocusChain()
-
+                continue
+            if self.ui.remote_folders:
+                self.assertIsInstance(widget, QtGui.QCheckBox)
+            # ROOT volume has no checkbox
+            elif treeiter.value().text(1) or self.ui.remote_folders:
+                self.assertIsInstance(widget, QtGui.QPushButton)
+            else:
+                self.assertIsInstance(widget, QtGui.QCheckBox)
+                widget = widget.nextInFocusChain()
+                self.assertIsInstance(widget, QtGui.QPushButton)
+            widget = widget.nextInFocusChain()
             treeiter += 1
-            item = treeiter.value()
 
     def test_widget_dict(self):
         """Ensure the widget_items dictionary is full."""
@@ -416,11 +451,11 @@ class FoldersPanelVolumesInfoTestCase(FoldersPanelTestCase):
         self.assertFalse(self.ui.ui.folders.header().isVisible())
 
         name = self.ui.ui.folders.headerItem().text(gui.FOLDER_NAME_COL)
-        self.assertEqual(name, gui.FOLDERS_COLUMN_NAME)
+        self.assertEqual(unicode(name), gui.FOLDERS_COLUMN_NAME)
         name = self.ui.ui.folders.headerItem().text(gui.SUBSCRIPTION_COL)
-        self.assertEqual(name, gui.ALWAYS_SUBSCRIBED)
+        self.assertEqual(unicode(name), gui.ALWAYS_SUBSCRIBED)
         name = self.ui.ui.folders.headerItem().text(gui.EXPLORE_COL)
-        self.assertEqual(name, gui.FOLDERS_COLUMN_EXPLORE)
+        self.assertEqual(unicode(name), gui.FOLDERS_COLUMN_EXPLORE)
 
     def test_share_publish_button(self):
         """When clicking the share/publish button, the proper url is opened."""
@@ -556,6 +591,8 @@ class FoldersPanelSubscriptionTestCase(FoldersPanelTestCase):
     def test_confirm_dialog_if_path_exists(self):
         """The confirmation dialog is correct."""
         self.patch(gui.os.path, 'exists', lambda path: True)
+        self.patch(gui.os.path, 'isdir', lambda path: True)
+        self.patch(gui, 'is_link', lambda path: False)
 
         # make sure the item is subscribed
         self.ui.is_processing = True
@@ -571,6 +608,48 @@ class FoldersPanelSubscriptionTestCase(FoldersPanelTestCase):
                          (self.ui, '', msg, buttons, gui.YES))
         self.assertEqual(FakedDialog.kwargs, {})
 
+    @defer.inlineCallbacks
+    def test_error_if_path_exists_as_file(self):
+        """The error dialog is correct."""
+        self.patch(gui.os.path, 'exists', lambda path: True)
+        self.patch(gui.os.path, 'isdir', lambda path: False)
+        self.patch(gui, 'is_link', lambda path: False)
+
+        # make sure the item is subscribed
+        self.ui.is_processing = True
+        self.set_item_checked()
+        self.ui.is_processing = False
+
+        yield self.ui.on_folders_itemChanged(self.item)
+
+        volume_path = self.item.volume_path
+        msg = gui.FOLDER_EXISTS_AS_FILE % {'folder_path': volume_path}
+        buttons = gui.CLOSE
+        self.assertEqual(FakedDialog.args,
+                         (self.ui, '', msg, buttons))
+        self.assertEqual(FakedDialog.kwargs, {})
+
+    @defer.inlineCallbacks
+    def test_error_if_path_exists_as_valid_link(self):
+        """The error dialog is correct."""
+        self.patch(gui.os.path, 'exists', lambda path: True)
+        self.patch(gui.os.path, 'isdir', lambda path: True)
+        self.patch(gui, 'is_link', lambda path: True)
+
+        # make sure the item is subscribed
+        self.ui.is_processing = True
+        self.set_item_checked()
+        self.ui.is_processing = False
+
+        yield self.ui.on_folders_itemChanged(self.item)
+
+        volume_path = self.item.volume_path
+        msg = gui.FOLDER_EXISTS_AS_FILE % {'folder_path': volume_path}
+        buttons = gui.CLOSE
+        self.assertEqual(FakedDialog.args,
+                         (self.ui, '', msg, buttons))
+        self.assertEqual(FakedDialog.kwargs, {})
+
     @defer.inlineCallbacks
     def test_confirm_dialog_if_path_does_not_exist(self):
         """The confirmation dialog is not created if not needed."""
@@ -674,11 +753,19 @@ class RemoteFoldersPanelSubscriptionTestCase(FoldersPanelSubscriptionTestCase):
 class BaseLocalFoldersTestCase(BaseTestCase):
     """Test suite for the class implementing the LocalFolders feature."""
 
+    fake_thread = True
+
     @defer.inlineCallbacks
     def setUp(self):
         self.path = u'not-existing-dir'
         self.expected_size = self.build_test_dir(self.path)
         self.queue = Queue.Queue()
+        self.kwargs['queue'] = self.queue
+        self.addCleanup(self.kwargs.pop, 'queue')
+
+        if self.fake_thread:
+            self.patch(gui, 'CalculateSize', FakedCalculateSize)
+
         yield super(BaseLocalFoldersTestCase, self).setUp()
 
     def build_test_dir(self, dir_path):
@@ -715,6 +802,8 @@ class BaseLocalFoldersTestCase(BaseTestCase):
 class CalculateSizeTestCase(BaseLocalFoldersTestCase):
     """Test suite for the CalculateSize thread implementation."""
 
+    fake_thread = False
+
     @defer.inlineCallbacks
     def setUp(self):
         yield super(CalculateSizeTestCase, self).setUp()
@@ -766,17 +855,6 @@ class CalculateSizeWithSymlinksTestCase(BaseLocalFoldersTestCase):
         return total_size
 
 
-class FakedCalculateSize(object):
-    """A faked CalculateSize thread."""
-
-    def __init__(self, *args, **kwargs):
-        self.started = False
-
-    def start(self):
-        """Fake start."""
-        self.started = True
-
-
 class CalculateSizeWithInvalidPath(CalculateSizeTestCase):
     """Test suite for the CalculateSize thread implementation."""
 
@@ -808,18 +886,14 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
     @defer.inlineCallbacks
     def setUp(self):
         yield super(FolderItemTestCase, self).setUp()
-        self.calculator = FakedCalculateSize(self.path, self.queue)
-        self.patch(gui, 'CalculateSize', lambda *a, **kw: self.calculator)
         self.values = ['foo', 'bar']
 
-        assert not self.calculator.started
-
     def test_no_params(self):
         """The creation with no params uses defaults."""
         item = gui.FolderItem()
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, None)
         self.assertEqual(item.volume_id, None)
         self.assertEqual(item.thread, None)
@@ -831,8 +905,10 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only values."""
         item = gui.FolderItem(values=self.values)
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), self.values[0])
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), self.values[1])
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)),
+                         self.values[0])
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)),
+                         self.values[1])
         self.assertEqual(item.path, None)
         self.assertEqual(item.volume_id, None)
         self.assertEqual(item.thread, None)
@@ -844,8 +920,8 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only a path."""
         item = gui.FolderItem(path=self.path)
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, self.path)
         self.assertEqual(item.volume_id, None)
         self.assertEqual(item.thread, None)
@@ -857,8 +933,8 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only a queue."""
         item = gui.FolderItem(queue=self.queue)
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, None)
         self.assertEqual(item.volume_id, None)
         self.assertEqual(item.thread, None)
@@ -870,8 +946,8 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only a volume_id."""
         item = gui.FolderItem(volume_id='yadda')
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, None)
         self.assertEqual(item.volume_id, 'yadda')
         self.assertEqual(item.thread, None)
@@ -883,8 +959,8 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only a volume_id."""
         item = gui.FolderItem(path=self.path, volume_id='yadda')
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, self.path)
         self.assertEqual(item.volume_id, 'yadda')
         self.assertEqual(item.thread, None)
@@ -896,21 +972,359 @@ class FolderItemTestCase(BaseLocalFoldersTestCase):
         """The creation with only a volume_id."""
         item = gui.FolderItem(path=self.path, queue=self.queue)
 
-        self.assertEqual(item.text(gui.LOCAL_SUBSCRIPTION_COL), '')
-        self.assertEqual(item.text(gui.LOCAL_SPACE_COL), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)), '')
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), '')
         self.assertEqual(item.path, self.path)
         self.assertEqual(item.volume_id, None)
-        self.assertEqual(item.thread, self.calculator)
+        self.assertIsInstance(item.thread, gui.CalculateSize)
         self.assertEqual(item.size, None)
         self.assertEqual(item.checkState(gui.LOCAL_SUBSCRIPTION_COL),
                          gui.UNCHECKED)
-
-        self.assertTrue(self.calculator.started)
+        # pylint: disable=E1101
+        self.assertTrue(item.thread.started)
 
 
-class LocalFoldersPanelTestCase(UbuntuOneBinTestCase):
-    """Test suite for the LocalFoldersPanel widget."""
+class BaseLocalFoldersPanelTestCase(UbuntuOneBinTestCase,
+                                    BaseLocalFoldersTestCase):
+    """Base test suite for the LocalFoldersPanel widget."""
 
     class_ui = gui.LocalFoldersPanel
 
-    # TODO: add the test suite (LP: #959690).
+    @defer.inlineCallbacks
+    def setUp(self):
+        self.patch(FakedControlPanelBackend, 'exposed_results',
+                   {'volumes_info': [],
+                    'account_info': ACCOUNT_INFO})
+        self.patch(gui, 'default_folders', self.default_folders)
+
+        self.timer = FakedTimer()
+        self.kwargs['timer'] = self.timer
+        self.addCleanup(self.kwargs.pop, 'timer')
+
+        yield super(BaseLocalFoldersPanelTestCase, self).setUp()
+
+        self.patch(self.ui.backend, 'validate_path_for_folder',
+                   self.validate_path)
+
+    def default_folders(self, user_home):
+        """A fixed default folders list."""
+        return [os.path.join(user_home, f) for f in DEFAULT_FOLDERS]
+
+    def validate_path(self, folder):
+        """A fixed path validator."""
+        return defer.succeed(u'invalid' not in folder.lower())
+
+    def assert_space_header_correct(self, total):
+        """Check that the 'used space' header is correct."""
+        total = '(%s)' % gui.humanize(total)
+        expected = gui.LOCAL_FOLDERS_SPACE_HEADER.format(space_total=total)
+        actual = self.ui.ui.folders.headerItem().text(gui.LOCAL_SPACE_COL)
+        self.assertEqual(expected, unicode(actual))
+
+    def assert_folder_added(self, path, volume_id=None, item=None):
+        """A entry for 'path' was added to the folder list."""
+        folders = self.ui.ui.folders
+
+        if path == MUSIC_PATH:
+            display_name = gui.MUSIC_DISPLAY_NAME
+        else:
+            display_name = path.replace(USER_HOME + os.path.sep, '')
+        if item is None:  # search for it
+            items = folders.findItems(display_name,
+                QtCore.Qt.MatchFixedString | QtCore.Qt.MatchCaseSensitive,
+                gui.FOLDER_NAME_COL)
+            self.assertEqual(len(items), 1)
+            item = items[0]
+        self.assertEqual(display_name,
+                         unicode(item.text(gui.LOCAL_SUBSCRIPTION_COL)))
+
+        self.assertEqual(item.path, path)
+        self.assertEqual(item.volume_id, volume_id)
+        # should be checked if the volume_id is not None
+        subscribed = item.checkState(gui.LOCAL_SUBSCRIPTION_COL) == gui.CHECKED
+        self.assertEqual(subscribed, volume_id is not None)
+
+        self.timer.timeout.emit()  # update sizes will be called
+
+        # if volume_id is not None, no 'space' is expected since is a cloud
+        # folder, and its size is accounted for in the quota used amount
+        size = gui.humanize(size_for_path(path)) if volume_id is None else ''
+        self.assertEqual(unicode(item.text(gui.LOCAL_SPACE_COL)), size)
+
+
+class LocalFoldersPanelTestCase(BaseLocalFoldersPanelTestCase):
+    """Test suite for the LocalFoldersPanel widget."""
+
+    def test_initial_state(self):
+        """Initial state is correct."""
+        self.assertIsInstance(self.ui.timer, gui.QtCore.QTimer)
+        self.assertFalse(self.ui.timer.isActive())
+
+        self.assertEqual(self.ui.ui.folders.topLevelItemCount(), 0)
+        self.assertFalse(self.ui.ui.offer_frame.isVisible())
+
+        headers = self.ui.ui.folders.header()
+        self.assertEqual(headers.resizeMode(gui.LOCAL_SUBSCRIPTION_COL),
+                         headers.Stretch)
+        self.assertEqual(headers.resizeMode(gui.LOCAL_SPACE_COL),
+                         headers.ResizeToContents)
+
+        header = self.ui.ui.folders.headerItem()
+        self.assertEqual(unicode(header.text(gui.LOCAL_SUBSCRIPTION_COL)),
+                         gui.LOCAL_FOLDERS_FOLDER_HEADER)
+
+        self.assertEqual(unicode(self.ui.ui.add_folder_button.text()),
+                         gui.FOLDERS_BUTTON_ADD_FOLDER)
+        self.assertEqual(self.ui.ui.add_folder_button.add_folder_func,
+                         self.ui.add_folder)
+        self.assertEqual(unicode(self.ui.ui.add_storage_button.text()),
+                         gui.GET_MORE_STORAGE)
+
+    @defer.inlineCallbacks
+    def test_is_processing_while_asking_info(self):
+        """The ui is processing while the contents are loaded."""
+
+        def check_account():
+            """The ui must be is_processing."""
+            self.assertTrue(self.ui.is_processing, 'ui must be processing')
+            return ACCOUNT_INFO
+
+        def check_volumes(with_storage_info):
+            """The ui must be is_processing."""
+            self.assertFalse(with_storage_info)
+            self.assertTrue(self.ui.is_processing, 'ui must be processing')
+            return FAKE_VOLUMES_ONLY_ROOT_INFO
+
+        self.patch(self.ui.backend, 'account_info', check_account)
+        self.patch(self.ui.backend, 'volumes_info', check_volumes)
+
+        yield self.ui.load()  # trigger the info request
+
+        self.assertEqual(self.ui.account_info, ACCOUNT_INFO)
+        self.assert_space_header_correct(ACCOUNT_INFO['quota_used'])
+
+    def test_is_not_processing_after_info_ready(self):
+        """The ui is not processing when contents are load."""
+        self.ui.process_info(FAKE_VOLUMES_ONLY_ROOT_INFO)
+
+        self.assertFalse(self.ui.is_processing)
+
+    @defer.inlineCallbacks
+    def test_info_is_requested_on_load(self):
+        """The volumes info is requested to the backend."""
+        yield self.ui.load()
+        self.assert_backend_called('account_info')
+        self.assert_backend_called('volumes_info', with_storage_info=False)
+
+    @defer.inlineCallbacks
+    def test_process_info(self):
+        """The volumes info is processed when ready."""
+        volumes_info = FAKE_VOLUMES_INFO
+        yield self.ui.process_info(volumes_info)
+
+        self.assertTrue(self.ui.timer.isActive())
+
+        # the following will execute update_sizes once
+        # will fill in all the path sizes for the already added folders
+        self.timer.timeout.emit()
+        volumes = []
+        for _, _, vols in volumes_info:
+            volumes.extend(vols)
+        volumes = sorted(volumes, key=operator.itemgetter('path'))
+
+        for volume in volumes:
+            if (volume['type'] != self.ui.backend.FOLDER_TYPE or
+                not bool(volume['subscribed'])):
+                # non-folders or unsubscribed should not appear in the tree
+                continue
+            self.assert_folder_added(volume['path'],
+                                     volume_id=volume['volume_id'])
+        for folder in gui.default_folders(USER_HOME):
+            valid = yield self.validate_path(folder)
+            if not valid:
+                continue  # invalid folders should not appear in the tree
+            self.assert_folder_added(folder, volume_id=None, item=None)
+
+        self.assert_space_header_correct(ACCOUNT_INFO['quota_used'])
+
+    @defer.inlineCallbacks
+    def test_process_info_twice(self):
+        """The volumes info can be processed twice."""
+        yield self.test_process_info()
+        yield self.test_process_info()
+
+    @defer.inlineCallbacks
+    def test_timer_is_not_started_if_no_volumes(self):
+        """If not volumes to show, timer is not started."""
+        self.patch(gui, 'default_folders', lambda *a, **kw: [])
+        yield self.ui.process_info([])
+        self.assertFalse(self.ui.timer.isActive())
+
+    @defer.inlineCallbacks
+    def test_on_folders_item_changed_subscription_column(self):
+        """If a new folder is subscribed, update header size."""
+        # processing only a root folder will force showing only default,
+        # unsubscribed folders
+        yield self.ui.process_info(FAKE_VOLUMES_ONLY_ROOT_INFO)
+
+        new_size = ACCOUNT_INFO['quota_used']
+        for i in xrange(self.ui.ui.folders.topLevelItemCount()):
+            item = self.ui.ui.folders.topLevelItem(i)
+            item.setCheckState(gui.LOCAL_SUBSCRIPTION_COL, gui.CHECKED)
+            new_size += size_for_path(item.path)
+            self.assert_space_header_correct(new_size)
+
+        # quota was not filled, so "get more storage" is not shown
+        available = ACCOUNT_INFO['quota_total'] - ACCOUNT_INFO['quota_used']
+        assert new_size <= available
+        self.assertFalse(self.ui.ui.offer_frame.isVisible())
+
+        # now, deselect each folder and confirm space column is correct
+        for i in xrange(self.ui.ui.folders.topLevelItemCount()):
+            item = self.ui.ui.folders.topLevelItem(i)
+            item.setCheckState(gui.LOCAL_SUBSCRIPTION_COL, gui.UNCHECKED)
+            new_size -= size_for_path(item.path)
+            self.assert_space_header_correct(new_size)
+
+        self.assert_space_header_correct(ACCOUNT_INFO['quota_used'])
+
+    @defer.inlineCallbacks
+    def test_on_folders_item_changed_unsubscribing(self):
+        """If a cloud folder was unsubscribed, do not update header."""
+        # patch update_sizes with sel.fail to ensure is not called
+        self.patch(self.ui, 'update_sizes', self.fail)
+
+        # process only root and a subscribed music folder
+        yield self.ui.process_info(FAKE_VOLUMES_MINIMAL_INFO)
+
+        item = self.ui.ui.folders.topLevelItem(0)
+        item.setCheckState(gui.LOCAL_SUBSCRIPTION_COL, gui.UNCHECKED)
+
+    @defer.inlineCallbacks
+    def test_on_folders_item_changed_space_column(self):
+        """If user clicked the 'space' column, don't update sizes."""
+        # patch update_sizes with sel.fail to ensure is not called
+        self.patch(self.ui, 'update_sizes', self.fail)
+
+        yield self.ui.process_info(FAKE_VOLUMES_INFO)
+
+        for i in xrange(self.ui.ui.folders.topLevelItemCount()):
+            item = self.ui.ui.folders.topLevelItem(i)
+            self.ui.on_folders_itemChanged(item, gui.LOCAL_SPACE_COL)
+            # size does not change
+            self.assert_space_header_correct(ACCOUNT_INFO['quota_used'])
+
+    @defer.inlineCallbacks
+    def test_going_over_quota(self):
+        """When subscribing a folder, alert the user if quota was exceeded."""
+        yield self.ui.process_info(FAKE_VOLUMES_INFO)
+
+        available = ACCOUNT_INFO['quota_total'] - ACCOUNT_INFO['quota_used']
+        self.patch(FakedCalculateSize, 'next_size', available + 1)
+
+        self.ui.add_folder(self.path)
+        self.ui.on_folder_created(self.path)
+
+        # quota was exceeeded, so "get more storage" should be shown
+        self.assert_space_header_correct(ACCOUNT_INFO['quota_total'] + 1)
+        self.assertTrue(self.ui.ui.offer_frame.isVisible())
+
+    # XXX: still pending to add tests about stopping the timer and the threads
+
+
+class LocalFoldersPanelAddFolderTestCase(BaseLocalFoldersPanelTestCase):
+    """Test suite for the add_folder method from LocalFoldersPanel."""
+
+    def test_folder_is_added(self, folder_path=None, suggested=None):
+        """Adding a folder adds it to the folder list."""
+        former_count = self.ui.ui.folders.topLevelItemCount()
+
+        if folder_path is None:
+            folder_path = self.path
+        self.ui.add_folder(folder_path, suggested=suggested)
+
+        new_count = self.ui.ui.folders.topLevelItemCount()
+        self.assertEqual(new_count, former_count + 1)
+        self.assert_folder_added(folder_path)
+
+    def test_add_music_folder_adds_the_folder(self):
+        """Adding the purchased music folder adds it to the folder list."""
+        self.test_folder_is_added(folder_path=MUSIC_PATH,
+            suggested=MUSIC_REAL_PATH)
+
+    def test_add_music_folder_adds_the_folder_with_suggested_path(self):
+        """Adding the purchased music folder adds it to the folder list."""
+        suggested_path = '~/' + MUSIC_REAL_PATH
+        self.test_folder_is_added(folder_path=MUSIC_PATH,
+            suggested=suggested_path)
+
+    def test_do_not_add_twice(self):
+        """The same folder will not be added twice."""
+        assert self.ui.ui.folders.topLevelItemCount() == 0
+
+        self.ui.add_folder(self.path)
+        self.ui.add_folder(self.path)
+
+        self.assert_folder_added(self.path)
+        self.assertEqual(1, self.ui.ui.folders.topLevelItemCount())
+
+    def test_do_not_call_backend(self):
+        """Adding a folder do not call the backend."""
+        self.ui.backend._called.clear()
+        self.ui.ui.add_folder_button.folderCreated.emit(self.path)
+
+        self.assertEqual(self.ui.backend._called, {})
+
+    def test_on_folder_created(self):
+        """When a folder was created, subscribe to it."""
+        self.ui.add_folder(self.path)
+
+        folders = self.ui.ui.folders
+        item = folders.topLevelItem(folders.topLevelItemCount() - 1)
+        assert item.checkState(gui.LOCAL_SUBSCRIPTION_COL) == gui.UNCHECKED
+
+        self.ui.ui.add_folder_button.folderCreated.emit(self.path)
+
+        subscribed = item.checkState(gui.LOCAL_SUBSCRIPTION_COL) == gui.CHECKED
+        self.assertTrue(subscribed)
+
+
+class LocalFoldersPanelApplyChangesTestCase(BaseLocalFoldersPanelTestCase):
+    """Test suite for the apply_changes method from LocalFoldersPanel."""
+
+    @defer.inlineCallbacks
+    def setUp(self):
+        yield super(LocalFoldersPanelApplyChangesTestCase, self).setUp()
+        self.changes_applied = defer.Deferred()
+        self.ui.backend._called.clear()
+        self.ui.changesApplied.connect(lambda:
+                                       self.changes_applied.callback(None))
+
+    @defer.inlineCallbacks
+    def test_apply_changes_calls_backend(self):
+        """When applying changes, the calls to the backend are made."""
+        yield self.ui.apply_changes()
+        yield self.changes_applied
+
+    @defer.inlineCallbacks
+    def test_apply_changes_stop_threads(self):
+        """When applying changes, threads are stopped."""
+        yield self.ui.apply_changes()
+        yield self.changes_applied
+
+    @defer.inlineCallbacks
+    def test_apply_changes_with_no_changes(self):
+        """If there are no changes to apply, the backend is not called."""
+        yield self.ui.apply_changes()
+        yield self.changes_applied
+        self.assertEqual(self.ui.backend._called, {})
+
+    def test_timer_is_stopped(self):
+        """When applying changes, timer is stopped."""
+        yield self.ui.apply_changes()
+        yield self.changes_applied
+        self.assertFalse(self.ui.timer.isActive())
+
+    # XXX: more tests are needed, to cover that only those newly added folders,
+    # and those newly subscribed or unsibscribed folders are modified by
+    # calling the backend by either create_folder or change_volume_settings
