@@ -81,6 +81,11 @@ class MainWindowTestCase(BaseTestCase):
             self.ui.ui.control_panel.ui.tab_widget.currentIndex(),
             self.ui.ui.control_panel.ui.tab_widget.indexOf(
                 self.ui.ui.control_panel.ui.folders_tab))
+        self.ui.switch_to("share_links")
+        self.assertEqual(
+            self.ui.ui.control_panel.ui.tab_widget.currentIndex(),
+            self.ui.ui.control_panel.ui.tab_widget.indexOf(
+                self.ui.ui.control_panel.ui.share_links_tab))
         self.ui.switch_to("devices")
         self.assertEqual(
             self.ui.ui.control_panel.ui.tab_widget.currentIndex(),
