@@ -205,3 +205,27 @@ class SyncDaemonClient(object):
     def refresh_volumes(self):
         """Refresh the volumes information from syncdaemon."""
         return self.proxy.refresh_volumes()
+
+    def sync_menu(self):
+        """Get the sync menu data from syncdaemon."""
+        return self.proxy.sync_menu()
+
+    def get_public_files(self):
+        """Trigger an action to get the list of public files."""
+        return self.proxy.get_public_files()
+
+    def set_public_files_list_handler(self, handler):
+        """Set the public files list handler."""
+        return self.proxy.connect_signal('PublicFilesList', handler)
+
+    def change_public_access(self, path, is_public):
+        """Change the access type of the file."""
+        return self.proxy.change_public_access(path, is_public)
+
+    def set_public_access_changed_handler(self, handler):
+        """Set the status handler function."""
+        return self.proxy.connect_signal('PublicAccessChanged', handler)
+
+    def set_public_access_change_error_handler(self, handler):
+        """Set the status handler function."""
+        return self.proxy.connect_signal('PublicAccessChangeError', handler)
