@@ -161,12 +161,22 @@ class MockSDClient(CallRecorder):
             'is_error': '', 'is_connected': 'True', 'is_online': '',
         }
         self.status_changed_handler = None
+        self.public_files_list_handler = None
+        self.public_access_changed_handler = None
         self.subscribed_folders = []
         self.subscribed_shares = []
         self.actions = []
         self.shares = []
         self.folders = []
         self.volumes_refreshed = False
+        self.menu_data = {'recent-transfers': (), 'uploading': ()}
+        self.publicfiles = [
+            {'path': '/home/file1', 'public_url': 'http:ubuntuone.com/asd123'},
+            {'path': '/home/file2', 'public_url': 'http:ubuntuone.com/qwe456'},
+        ]
+        self.public_access_info = {
+            'public_url': 'http:ubuntuone.com/zxc789'
+        }
 
     def get_throttling_limits(self):
         """Return the sample speed limits."""
@@ -326,6 +336,28 @@ class MockSDClient(CallRecorder):
         self.volumes_refreshed = True
         return defer.succeed(None)
 
+    def sync_menu(self):
+        """Return the sync menu data."""
+        return self.menu_data
+
+    def get_public_files(self):
+        """Trigger the action to get the public files."""
+        if self.public_files_list_handler is not None:
+            self.public_files_list_handler(self.publicfiles)
+
+    def set_public_files_list_handler(self, handler):
+        """Return the handler to be called for the public files list."""
+        self.public_files_list_handler = handler
+
+    def change_public_access(self, path, is_public):
+        """Change the type access of a file."""
+        if self.public_access_changed_handler is not None and is_public:
+            self.public_access_changed_handler(self.public_access_info)
+
+    def set_public_access_changed_handler(self, handler):
+        """Return the handler to be called when a access type change."""
+        self.public_access_changed_handler = handler
+
 
 class MockReplicationClient(CallRecorder):
     """A mock replication_client module."""
@@ -491,6 +523,32 @@ class BackendCredentialsTestCase(BackendBasicTestCase):
         self.assertEqual(creds1, creds2)
         self.assertEqual(self.be.login_client._called['find_credentials'], 1)
 
+    @inlineCallbacks
+    def test_credentials_are_cached_slow(self):
+        """The credentials are still cached when find_credentials is slow."""
+        dq = defer.DeferredQueue()
+
+        @inlineCallbacks
+        def delayed_find_creds():
+            """Function with externally controlled return time."""
+            fake_creds = yield dq.get()
+            defer.returnValue(fake_creds)
+
+        self.patch(self.be.login_client, 'find_credentials',
+                   delayed_find_creds)
+
+        # Ensure that the second call to get_credentials occurs before
+        # the first one gets a result from find_credentials by leaving
+        # the queue empty until both are called:
+
+        d1 = self.be.get_credentials()
+        d2 = self.be.get_credentials()
+        dq.put("Blizzard")
+        dq.put("Dilly Bar")
+        creds1 = yield d1
+        creds2 = yield d2
+        self.assertEqual(creds1, creds2)
+
     @inlineCallbacks
     def test_clear_credentials(self):
         """The credentials can be cleared."""
@@ -1361,8 +1419,8 @@ class BackendSyncStatusTestCase(BackendBasicTestCase):
         self.assertEqual(self.was_disabled, self.be.file_sync_disabled)
 
     def test_status_changed(self):
-        """The file_sync_status is the status changed handler."""
-        self.be.status_changed_handler = self._set_called
+        """The status changed handler is called with a processed status."""
+        self.be.add_status_changed_handler(self._set_called)
         status = {'name': 'foo', 'description': 'bar', 'is_error': '',
                   'is_connected': '', 'is_online': '', 'queues': ''}
         self.be.sd_client.status_changed_handler(status)
@@ -1372,12 +1430,56 @@ class BackendSyncStatusTestCase(BackendBasicTestCase):
 
     def test_invalid_status_type(self):
         """Check that the method return None with invalid types."""
-        self.be.status_changed_handler = self._set_called
+        self.be.add_status_changed_handler(self._set_called)
         status = 3
         self.be.sd_client.status_changed_handler(status)
 
         self.assertEqual(self._called, ((None,), {}))
 
+    def test_add_same_status_handler_twice(self):
+        """Should ignore adding same handler twice."""
+        self.call_count = 0
+
+        def inc_count(status):
+            """Fake Callback."""
+            self.call_count += 1
+
+        self.addCleanup(delattr, self, "call_count")
+
+        self.be.add_status_changed_handler(inc_count)
+        self.be.add_status_changed_handler(inc_count)
+
+        status = {'name': 'foo', 'description': 'bar', 'is_error': '',
+                  'is_connected': '', 'is_online': '', 'queues': ''}
+        self.be.sd_client.status_changed_handler(status)
+
+        self.assertEqual(self.call_count, 1)
+        self.assertEqual(self.be._status_changed_handlers,
+                         [inc_count])
+
+    def test_add_two_status_handlers(self):
+        """Should accept and call multiple status handlers."""
+        self.call_count_a = 0
+        self.call_count_b = 0
+
+        def inc_a(status):
+            """Fake status handler #1"""
+            self.call_count_a += 1
+
+        def inc_b(status):
+            """Fake status handler #2"""
+            self.call_count_b += 1
+
+        self.addCleanup(delattr, self, "call_count_a")
+        self.addCleanup(delattr, self, "call_count_b")
+
+        self.be.add_status_changed_handler(inc_a)
+        self.be.add_status_changed_handler(inc_b)
+
+        self.be.sd_client.status_changed_handler({})
+        self.assertEqual(self.call_count_b, 1)
+        self.assertEqual(self.call_count_a, 1)
+
 
 class BackendSyncStatusIfDisabledTestCase(BackendSyncStatusTestCase):
     """Syncdaemon state for the backend when file sync is disabled."""
@@ -1697,3 +1799,47 @@ class BackendFileSyncSettingsTestCase(BackendBasicTestCase):
 
         result = yield self.be.file_sync_settings_info()
         self.assertEqual(self.default_settings, result)
+
+    @inlineCallbacks
+    def test_sync_menu(self):
+        """Check that we get the right data to create the menu."""
+        result = yield self.be.sync_menu()
+        expected = {'recent-transfers': (), 'uploading': ()}
+        self.assertEqual(result, expected)
+
+    @inlineCallbacks
+    def test_get_public_files(self):
+        """Check that we get list of public files."""
+        data = []
+
+        def get_public_files_handler(publicfiles):
+            """Callback for get_public_files."""
+            data.append(publicfiles)
+
+        self.be.set_public_files_list_handler(get_public_files_handler)
+        yield self.be.get_public_files()
+        expected = [[{'path': '/home/file1',
+                      'public_url': 'http:ubuntuone.com/asd123'},
+                     {'path': '/home/file2',
+                      'public_url': 'http:ubuntuone.com/qwe456'}]]
+        self.assertEqual(expected, data)
+
+    @inlineCallbacks
+    def test_get_shares(self):
+        """Check that we get the list of shares."""
+        result = yield self.be.get_shares()
+        self.assertEqual(result, [])
+
+    @inlineCallbacks
+    def test_change_public_access_set_public(self):
+        """Check that we get a notification when the access type change."""
+        data = []
+
+        def public_access_change_handler(info):
+            """Callback for get_public_files."""
+            data.append(info)
+
+        self.be.set_public_access_changed_handler(public_access_change_handler)
+        yield self.be.change_public_access('file1', True)
+        expected = {'public_url': 'http:ubuntuone.com/zxc789'}
+        self.assertEqual([expected], data)
