@@ -21,7 +21,7 @@ from urlparse import urlparse, parse_qs
 from twisted.internet import defer
 from twisted.web import resource
 
-from ubuntu_sso.utils.webclient.tests import BaseMockWebServer
+from ubuntuone.devtools.testing.txwebserver import HTTPWebServer
 
 from ubuntuone.controlpanel.tests import TestCase
 from ubuntuone.controlpanel.web_client import (
@@ -73,10 +73,10 @@ class MockResource(resource.Resource):
         return self.contents
 
 
-class MockWebServer(BaseMockWebServer):
+class MockWebServer(HTTPWebServer):
     """A mock webserver for the webclient tests."""
 
-    def get_root_resource(self):
+    def __init__(self):
         """Get the root resource with all the children."""
         root = resource.Resource()
         devices_resource = MockResource()
@@ -86,8 +86,7 @@ class MockWebServer(BaseMockWebServer):
         unauthorized = resource.ErrorPage(resource.http.UNAUTHORIZED,
                                           "Unauthrorized", "Unauthrorized")
         root.putChild("unauthorized", unauthorized)
-
-        return root
+        super(MockWebServer, self).__init__(root)
 
 
 class WebClientTestCase(TestCase):
@@ -99,6 +98,7 @@ class WebClientTestCase(TestCase):
     def setUp(self):
         yield super(WebClientTestCase, self).setUp()
         self.ws = MockWebServer()
+        self.ws.start()
         self.addCleanup(self.ws.stop)
         self.base_iri = self.ws.get_iri()
 
